package com.hncy.datatrans.server.pzhdata.excel;

import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.hncy.datatrans.server.pzhdata.entity.Area;
import com.hncy.datatrans.server.pzhdata.entity.BasicSchoolInfo;
import com.hncy.datatrans.server.pzhdata.mapper.BasicSchoolInfoMapper;
import com.hncy.datatrans.server.pzhdata.utils.IvanTool;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @Author Ivan
 * @Description
 * @Date 2020/9/18 11:05
 */
@Slf4j
public class SchoolExcelDataListener extends AnalysisEventListener<SchoolExcelData> {
    /**
     * 每隔500条存储数据库，实际使用中可以3000条，然后清理list ，方便内存回收
     */
    private static final int BATCH_COUNT = 500;

    //因为ExcelListener不能交给spring进行管理，需要自己new，不能注入其他对象 不能实现数据库操作
    public BasicSchoolInfoMapper schoolInfoMapper;

    public List<Area> areas;

    //存储数据
    List<SchoolExcelData> dataList = new ArrayList<>();

    //无参构造函数
    public SchoolExcelDataListener() {}
    //有参构造函数
    public SchoolExcelDataListener(BasicSchoolInfoMapper schoolInfoMapper, List<Area> areas) {
        this.schoolInfoMapper = schoolInfoMapper;
        this.areas = areas;
    }



    //读取excel内容，一行一行进行读取
    @Override
    public void invoke(SchoolExcelData schoolExcelData, AnalysisContext analysisContext) {
        if(schoolExcelData == null) {
            System.out.println("文件数据为空...");
            throw new RuntimeException("文件数据为空...");
        }
//        System.out.println(conflictExcelData);
        dataList.add(schoolExcelData);
        // 达到BATCH_COUNT了，需要去存储一次数据库，防止数据几万条数据在内存，容易OOM
        if (dataList.size() >= BATCH_COUNT) {
            //保存到数据库
            saveData();
            // 存储完成清理 list
            dataList.clear();
        }
    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext analysisContext) {
        // 这里也要保存数据，确保最后遗留的数据也存储到数据库
        saveData();
        log.info("所有数据解析完成！");
    }

    /**
     * 加上存储数据库
     */
    private void saveData() {
        //日期格式
        log.info("{}条数据，开始存储数据库！", dataList.size());
        //处理数据
        if (dataList!=null&&dataList.size()>0){
            List<BasicSchoolInfo> schoolInfos = new ArrayList<>();
            for (SchoolExcelData data : dataList){
                BasicSchoolInfo schoolInfo = new BasicSchoolInfo();

                //复制相同的字段
                BeanUtils.copyProperties(data,schoolInfo);

                //主键id
                schoolInfo.setId(IvanTool.getId());
                //电话
                if (data.getPrincipalPhone().length()!=11){
                    schoolInfo.setPrincipalPhone(data.getQuHao()+"-"+data.getPrincipalPhone());
                }
                //学校类型
                schoolInfo.setSchoolTypeCode("51");
                schoolInfo.setSchoolTypeName("特殊教育");
                //学生人数
                schoolInfo.setStudentNumber(Integer.parseInt(data.getStudentNumber()));
                schoolInfo.setCreateBy("Admin");
                schoolInfo.setCreateTime(new Date());
                schoolInfo.setUpdateBy("Admin");
                schoolInfo.setUpdateTime(new Date());

                try {
                    //处理特殊类型字段：辖区
                    //辖区
                    String area = data.getArea();
                    if (area!=null) {
                        if (area.contains("办事处")){
                            area = area.substring(0, area.indexOf("办事处"));
                        }
                        if (areas!=null&&areas.size()>0){
                            for (Area areaBase : areas){
                                if (area.equals(areaBase.getAreaName())){
                                    //辖区名称
                                    schoolInfo.setArea(areaBase.getAreaName());
                                    //辖区id
                                    schoolInfo.setGridId(areaBase.getId());
                                    break;
                                }
                            }
                        }
                        //网格
                        if (schoolInfo.getGridId()==null){
                            String area1 = null;
                            String gridId = null;
                            switch (data.getCounty()){
                                case "米易县" :
                                    area1 = "米易县";
                                    gridId = "7aMBFTsY";
                                    break;
                                case "钒钛新城" :
                                    area1 = "钒钛新城";
                                    gridId = "9TSpn4dA";
                                    break;
                                case "仁和区" :
                                    area1 = "仁和区";
                                    gridId = "XgLd3aTV";
                                    break;
                                case "盐边县" :
                                    area1 = "盐边县";
                                    gridId = "Z5mu6fGN";
                                    break;
                                case "东区" :
                                    area1 = "东区";
                                    gridId = "h18lsSgo";
                                    break;
                                case "西区" :
                                    area1 = "西区";
                                    gridId = "jhRxqEQp";
                                    break;
                            }
                            schoolInfo.setArea(area1);
                            schoolInfo.setGridId(gridId);
                        }
                    }
                } catch (Exception e){
                    e.printStackTrace();
                }

                //添加一条数据
                schoolInfos.add(schoolInfo);
            }
            //保存数据
            int result = schoolInfoMapper.addSchools(schoolInfos);
            log.info(result + "条数据保存成功！");
        }
    }
}
