package com.hncy.datatrans.server.pzhdata.utils;

import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.util.HashMap;
import java.util.Map;

/**
 * @Author Ivan
 * @Description
 * @Date 2020/8/25 11:25
 */
@Data
public class CommonResult {

    @ApiModelProperty(value = "返回码")
    private Integer code;

    @ApiModelProperty(value = "返回消息")
    private String message;

    @ApiModelProperty(value = "返回数据")
    private Map<String, Object> data = new HashMap<>();

    //把构造方法私有
    private CommonResult() {}

    /**
     * @author Ivan
     * @date 2020/7/19 22:31
     * @Description: 成功静态方法
     */
    public static CommonResult success() {
        CommonResult result = new CommonResult();
        result.setCode(200);
        result.setMessage("操作成功!");
        return result;
    }

    /**
     * @author Ivan
     * @date 2020/7/19 22:31
     * @Description: 失败静态方法
     */
    public static CommonResult fail() {
        CommonResult result = new CommonResult();
        result.setCode(-1);
        result.setMessage("操作失败!");
        return result;
    }

    /**
     * @author Ivan
     * @date 2020/7/19 22:31
     * @Description: 设置返回消息
     */
    public CommonResult message(String message){
        this.setMessage(message);
        return this;
    }

    /**
     * @author Ivan
     * @date 2020/7/19 22:31
     * @Description: 设置返回状态码
     */
    public CommonResult code(Integer code){
        this.setCode(code);
        return this;
    }

    /**
     * @author Ivan
     * @date 2020/7/19 22:31
     * @Description: 设置返回数据的键值对
     */
    public CommonResult data(String key, Object value){
        this.data.put(key, value);
        return this;
    }

    /**
     * @author Ivan
     * @date 2020/7/19 22:31
     * @Description: 设置返回数据
     */
    public CommonResult data(Map<String, Object> map){
        this.setData(map);
        return this;
    }
}
